from langchain_openai import ChatOpenAI
from langchain.agents import initialize_agent
from ..agents.movie_agent import MovieAgent
from ..tasks.ha_common_task import HaCommonTask
import os
from ..model.youtube_info import YoutubeInfo
from ..tasks.baas_clsname_task import BassClsNameTask
from time import sleep
from datetime import datetime, date, timedelta
        

class HaYoutubeAgent:
    def __init__(self):
        self.task_common = HaCommonTask()
        self.task_baas = BassClsNameTask()
        self.class_name_master = "YoutubeMaster"
        self.class_name_detail = "YoutubeDetail_"
        self.class_name_daily = "YoutubeMasterDaily"
        


    def get_youtube_master_info(self, order_key:str="author"):    
        data = self.task_baas.basicOrderGet(class_name=self.class_name_master, orderKey=order_key)
        try:
            result = data.json()
            return result["results"]
        except Exception as e:
            print(e)
            return []

    def get_yt(self,url:str):
        try:
            yt = self.task_common.get_youtube_info(url)
            return yt
        except Exception as e:
            if "403" in str(e):
            # 403 에러시 토큰 재생성
                self.task_common.regenerate_network_token(url=url)
                sleep(10)
            print(e)
            return None

    def get_yt_info(self, url:str):
        yt = self.get_yt(url)
        info = YoutubeInfo()
        data = info.change_model(yt=yt) 
        return data

    def save_yt_master(self, url:str, data:dict):
        yt = self.get_yt(url=url)
        channel_id = yt.channel_id
        check = self.task_baas.exist_key_date(class_name=self.class_name_master, keyName="channel_id", data=channel_id) 
        if check:
            print(f"{yt.author} [{channel_id}]이미 존재합니다.")
        else:
            res = self.task_baas.basicPost(class_name=self.class_name_master, data=data)
            check = res.status_code == 200
            data["check"] = check
            print(f"{yt.author} [{channel_id}]저장 되었습니다.")
        return data
    
    def save_yt_detail(self, video_urls:list, data:dict, limit:int=20):
        list = []
        for youtube_url in video_urls:    
            vedios = self.task_common.get_channel_urls(url=youtube_url, limit=limit, sleep=2)
            for index, video in enumerate(vedios):
                sleep(2)

                data = self.get_yt_info(video)
                channel_id = data["channel_id"]
                try:

                    object_id = self.task_baas.get_object_id(class_name=self.class_name_master, keyName="channel_id", data=channel_id)
                    class_name = self.class_name_detail + object_id
                    
                    if index == 0:
                        ## 삭제하고 
                        self.task_baas.delete_all_class(class_name=class_name)
                    
                    ## 다시 저장
                    res = self.task_baas.basicPost(class_name=class_name, data=data)
                    check = res.status_code == 200
                    data["check"] = check
                    print(f"{data['author']} || [{class_name}] {data['title']} 저장 되었습니다.  [{data['publish_date']}]")
                    list.append({ "objectId":object_id, "description":data['description'], "title":data['title'], "video_length":data['video_length'], "publish_date":data['publish_date'], 
                                    "thumbnail_url":data['thumbnail_url'], "watch_url":data['watch_url'], "check":check})
                except Exception as e:
                    print(e)
                    print(f"object_id : {object_id} || class_name : {class_name}")
                    continue    
        return list            

     # 유튜브 영상으로 부터 텍스트를 추출
    def working_youtube_to_text(self, youtube_url:str):
        agent_movie = MovieAgent(youtube_url=youtube_url)
        agent_movie.working_youtube_to_text()


    def load_file(self, youtube_url:str):
        agent_movie = MovieAgent(youtube_url=youtube_url)
        agent_movie.working_youtube_to_text()
        res = agent_movie.load_file()
        # print(f"res: {res}")
        return res

    # # 텍스트가 있으면 Back4app에 저장
    def save_youtube_channel_info(self, app_name:str, video_url:str, limit:int=10, sort_by:str="newest")->list:
        common_task = HaCommonTask()
        baas_task = BassClsNameTask(app_name=app_name)
        class_name = "YoutubeInfo"

        videos = common_task.get_channel_urls(video_url, limit=limit, sort_by=sort_by)
        list = []
        for video in videos:
            yt = common_task.get_youtube_info(video)

            transcribe_text = self.load_file(video)

            youtube_info = YoutubeInfo()
            yt.transcribe_text = transcribe_text
            yt.summary_text = ''
            yt.embedding_text = ''
            
            data = youtube_info.change_model(yt)

            check = baas_task.exist_key_date(class_name=class_name, keyName="video_id", data=data["video_id"])
            if check:
                print("이미 존재합니다.")
                data["check"] = True
                list.append({"title":data['title'], "video_length":data['video_length'], "publish_date":data['publish_date'], 
                             "thumbnail_url":data['thumbnail_url'], "watch_url":data['watch_url'], "check":True})
            else:
                res = baas_task.basicPost(class_name=class_name, data=data)
                check = res.status_code == 200
                data["check"] = check
                list.append({"title":data['title'], "video_length":data['video_length'], "publish_date":data['publish_date'], 
                             "thumbnail_url":data['thumbnail_url'], "watch_url":data['watch_url'], "check":check})
        return list
    


    def check_range_date(self, publish_date:str):
        today = date.today() 
        one_week = timedelta(days=7)
        one_week_ago = today - one_week

        ref_date = datetime.strptime(f"{one_week_ago}", "%Y-%m-%d")
        target_date = datetime.strptime(publish_date, "%Y-%m-%d %H:%M:%S")
        check = ref_date < target_date
        return check
    
    # 전부삭제하고 새로데이터 추가
    def cron_youtube_remove_and_add(self):
        master_data = self.get_youtube_master_info(order_key="-publish_date")
        for master in master_data:
            try:
                url = master['watch_url']
                data = self.get_yt_info(url)

                self.save_yt_master(url=url, data=data)
                video_urls = [url]
                list_data = self.save_yt_detail(video_urls=video_urls, data=data, limit=10)
                
                if list_data is not None and len(list_data) > 0:
                    recent_data = list_data[0]
                    video_id = self.task_common.get_audio_id(url=url)
                    update_data = {
                                "title": recent_data['title'],
                                "thumbnail_url": recent_data['thumbnail_url'],
                                "watch_url": recent_data['watch_url'],
                                "publish_date": recent_data['publish_date'],
                                "video_length": recent_data['video_length'],
                                "description": recent_data['description'],
                                "video_id": video_id,
                            }
                    self.task_baas.basicPut(class_name=self.class_name_master, object_id=recent_data['objectId'], data=update_data)
            except Exception as e:
                if "403" in str(e): 
                    # 403 에러시 토큰 재생성
                    self.task_common.regenerate_network_token(url=url)
                    sleep(10)
                print(f"[cron_youtube_remove_and_add] error: {e}")


    # 오늘 날짜 유튜브 채널 정보 저장
    def cron_youtube_daily(self):
        ## 삭제로직 먼저 실행
        self.task_baas.delete_all_class(class_name=self.class_name_daily)

        ## YoutubeMaster 조회 후 저장
        datas = self.get_youtube_master_info()
        for index, data in enumerate(datas):
            author = data['author']
            objectId = data['objectId']
            updateObjectId = data['objectId']
            publish_date = data['publish_date']
            check = self.check_range_date(publish_date=publish_date)
            # print(f"체크 {author} {index+1} => {check} || {publish_date}")
            if check:
                sleep(2)
                class_name_current = self.class_name_detail+objectId
                res = self.task_baas.basicOrderGet(class_name=class_name_current, orderKey="-publish_date")
                if res.status_code == 200:  
                    results = res.json()['results']
                    if len(results) > 0:
                        data = results[0]
                        ### 아래 객체에서 제거해야 하는 항목 처리
                        data.pop('objectId',None)
                        data.pop('createdAt',None)
                        data.pop('updatedAt',None)
                        data['cron_date'] = datetime.now().strftime("%Y-%m-%d")
                     
                        res_save = self.task_baas.basicPost(class_name=self.class_name_daily, data=data)
                        print(f"res_save: {res_save.status_code} || {res_save.content}")
                        if res_save.status_code == 201:
                            print(f"저장 완료 {author} {index+1}")
                            # master 업데이트
                            video_id = self.task_common.get_audio_id(url=data['watch_url'])
                            update_data = {
                                "title": data['title'],
                                "thumbnail_url": data['thumbnail_url'],
                                "watch_url": data['watch_url'],
                                "publish_date": publish_date,
                                "description": data['description'],
                                "video_length": data['video_length'],
                                "video_id": video_id,
                            }
                            self.task_baas.basicPut(class_name=self.class_name_master, object_id=updateObjectId, data=update_data)
                    
      
                    else:
                        print(results)
                else:
                    print(res.content)


    # 신규 채널 등록
    def save_newest_channel(self, url:str):
        data = self.get_yt_info(url)

        self.save_yt_master(url=url, data=data)
        video_urls = [url]
        list_data = self.save_yt_detail(video_urls=video_urls, data=data, limit=20)         
        return list_data
    

    def get_youtube_detail_recent(self, object_id:str):
        class_name_detail = self.class_name_detail+object_id
        res = self.task_baas.basicOrderGet(class_name=class_name_detail, orderKey="-publish_date")
        if res.status_code == 200:
            results = res.json()['results']
            return results
        else:
            return []


    def delete_youtube_master_and_detail(self, object_id:str):
        detail_class_name = self.class_name_detail+object_id
        # 2. delete detail
        self.task_baas.delete_all_class(class_name=detail_class_name)
        # 2. delete master
        res = self.task_baas.basicDelete(class_name=self.class_name_master, object_id=object_id)
        return res

    def delete_youtube_detail(self, class_id:str, movie_id:str):
        detail_class_name = self.class_name_detail+class_id
        res = self.task_baas.basicDelete(class_name=detail_class_name, object_id=movie_id)
        return res


    def get_youtube_daily(self):
        res = self.task_baas.basicOrderGet(class_name=self.class_name_daily, orderKey="-views")
        if res.status_code == 200:
            results = res.json()['results']
            return results
        else:
            return []
        
    def get_youtube_master_order_publish_date(self):
        res = self.task_baas.basicOrderGet(class_name=self.class_name_master, orderKey="-publish_date")
        if res.status_code == 200:
            results = res.json()['results']
            return results
        else:
            return []    
        
    def add_youtube_channel(self, video_url:str):
        data = self.get_yt_info(video_url)
        self.save_yt_master(url=video_url, data=data)
        video_urls = [video_url]
        list_data = self.save_yt_detail(video_urls=video_urls, data=data, limit=10)         
        return list_data

