
import os
import json
from ..tasks.daglo_task import DagloTask
from ..tasks.youtube_task import YoutubeTask
from ..tasks.ha_common_task import HaCommonTask
from ..tasks.file_task import FileTask
from ..ha_constant import HaConstant
from ..model.youtube_info import YoutubeInfo
from ..tasks.baas_clsname_task import BassClsNameTask
from time import sleep
from datetime import datetime
from ..agents.ha_youtube_agent import HaYoutubeAgent
from ..agents.naver_agent import NaverAgent
from ..tasks.rag_task import RAGTask
from ..tasks.whisper_task import WhisperTask

os.environ['KMP_DUPLICATE_LIB_OK']='True'

class WhisperAgent:
    def __init__(self):
        self.agent_youtube = HaYoutubeAgent()
        self.agent_naver = NaverAgent()
        self.task_whisper = WhisperTask()
        self.task_youtube = YoutubeTask()
        self.task_common = HaCommonTask()
        self.task_baas = BassClsNameTask()
        self.task_file = FileTask()
        self.task_rag = RAGTask()
        self.embed_dir = HaConstant.embed_dir
        self.clss_name_daglo = "DagloWork"
        self.clss_name_master = "YoutubeMaster"
        self.clss_name_detail = "YoutubeDetail_"


    def step1_youtube_to_mp3(self, url:str):
        self.task_youtube.open_and_get_audio(url)

    ## 1. 유튜브 다운로드 해서 mp3 파일이 생성이 되면
    ## 2. whisper 모델로 텍스트 추출
    ## 3. 다글로 rid로 체킹 

    def run(self, url:str, exist_txt:bool=False):
        audio_id = self.task_common.get_audio_id(url)
        print(f"[run] audio_id: {audio_id}")
        exist_mp3 = self.task_file.exist_audio_file(audio_id=audio_id)
        print(f"exist_txt: {exist_txt} || {self.embed_dir}/{audio_id}.txt")

        if not exist_mp3 and not exist_txt:            
            self.step1_youtube_to_mp3(url)
            print("step1_youtube_to_mp3")

        if not exist_txt:
            exist_txt = self.task_whisper.transcribe_chunks(audio_id=audio_id, chunk_folder=HaConstant.audio_dir, destination=f"{self.embed_dir}/{audio_id}.txt")
               
        while True:
            if exist_txt:
                print(f"exist_txt: {exist_txt}")
                self.task_file.delete_audio_file(audio_id=audio_id)
                break
            else:
                print(f"sleep exist_txt: {exist_txt} {audio_id}")
                exist_txt = self.task_whisper.transcribe_chunks(audio_id=audio_id, chunk_folder=HaConstant.audio_dir, destination=f"{self.embed_dir}/{audio_id}.txt")
                sleep(10)
   
        print(f"run end")

    # 영상 처리 대기건 처리용
    def action_wait_daglo_work(self, url):
        audio_id = self.task_common.get_audio_id(url)
        res = self.task_baas.get_key_datas(class_name=self.clss_name_daglo, keyName="audio_id", data=audio_id)
        step = res['results'][0]['step']
        rid = res['results'][0]['rid']
        if step == 'RECIEVED_RID':
            self.get_daglo_res_and_after(url=url, rid=rid, audio_id=audio_id)

    # rid 받았으나 이후 프로세스 진행이 안된경우에 돌리는 함수
    def get_daglo_res_and_after(self, url:str, rid:str, audio_id:str):
        pass
        # yt_info = self.task_common.get_youtube_info(url)
        # youtube_info = YoutubeInfo()
        # data = youtube_info.change_model(yt=yt_info)
        # exist_txt = self.task_file.exist_text_file(audio_id=audio_id)
        # print(f"get_daglo_res_and_after start exist_txt: {exist_txt}")
        # while not exist_txt and rid is not None:
        #     res = self.step3_rid_check(rid=rid, audio_id=audio_id, youtube_info=data)
        #     print(f"step3_rid_check: {res}")
            
        #     sleep(30)
        #     if res and self.step4_daglo_check_and_write_txt(res, audio_id=audio_id, rid=rid, youtube_info=data):
        #         print("step4_daglo_check_and_write_txt")
        #         # 폴더내 mp3 파일 삭제
        #         self.task_file.delete_audio_file(audio_id=audio_id)
        #         break
        #     else:
        #         sleep(10)
        # print(f"get_daglo_res_and_after end")

    def get_yt_info(self, url):
        yt_info = self.agent_youtube.get_yt(url)
        youtube_info = YoutubeInfo()
        data = youtube_info.change_model(yt=yt_info)
        return data      
    
    def save_or_update_report(self, class_name=None, audio_id=None, data=None):
        try:
            object_id =  self.task_baas.get_object_id(class_name=class_name, keyName="video_id", data=audio_id)
            print(f"video_id [object_id]: {object_id}")
            if object_id is not None:
                # 업데이트
                if 'createdAt' in data:
                    del data['createdAt']
                if 'updatedAt' in data:
                    del data['updatedAt']
                # if audio_id != video_id:
                #     video_id = audio_id
                #     data['video_id'] = audio_id
                object_id = data['objectId']
                # print(f"save_or_update_report UPDATE data: {data}")
                res =self.task_baas.basicPut(class_name=class_name, object_id=object_id, data=data)
                print(f"save_or_update_report UPDATE res: {class_name} {object_id} || {data['objectId']} vs {data['video_id']} || {res.status_code} {res.content}")
                # print(f"save_or_update_report UPDATE data: {data}")
            elif object_id is None:               # 신규 저장
                if 'objectId' in data:
                    del data['objectId']
                if 'createdAt' in data:
                    del data['createdAt']
                if 'updatedAt' in data:
                    del data['updatedAt']    
                
                if 'objectId' not in data:
                    res = self.task_baas.basicPost(class_name=class_name, data=data)
                    print(f"save_or_update_report INSERT res: {class_name} {res.status_code} {res.content}")
        except Exception as e:
            print(f"save_or_update_report Error: {e}")

    def get_detail_object_id_by_video_id(self, class_name:str, video_id:str):
        object_id = self.task_baas.get_object_id(class_name=class_name,keyName="video_id", data=video_id)
        return object_id

    def get_class_name(self, channel_id):
        object_id = self.task_baas.get_object_id(class_name=self.clss_name_master,keyName="channel_id", data=channel_id)

        class_name = self.clss_name_detail+object_id
        return class_name 

    def rag_run(self, url:str):
        # try:
        
        audio_id = self.task_common.get_audio_id(url)

        if audio_id is None:
            return None

        data = self.get_yt_info(url)
        
        channel_id = data['channel_id']
        res = self.task_baas.get_key_datas(class_name=self.clss_name_master, keyName="channel_id", data=channel_id)
        prompt_msg = "내용안에서 언급된 책이 있으면 책 제목을 정리해서 알려줘"
        # print(f">>> res: {res}")
        if res['results']:
            data = res['results'][0]
            print(f"data: {data}")
            prompt_msg = data['prompt_msg']

        class_name = self.get_class_name(channel_id)
        # is_rag = self.task_baas.get_key_datas(class_name=object_id, keyName="is_rag", data=True)        
        # if is_rag:
        #     return True
        texts = ["내용을 요약해서 정리해줘", 
                prompt_msg, # "내용안에서 언급된 책이 있으면 책 제목을 정리해서 알려줘",
                "내용안에서 언급된 사람이 있으면 이름을 정리해서 알려줘",
                "내용안에서 발췌해서, 유튜브 조회수 100만 가능한 제목으로 10개 만들어줘", 
                "내용안에서 발췌해서, 유튜브 조회수 100만 가능한 해시태그 10개 만들어줘", 
                "내용을 4지선다형 객관식 문제로 10개 만들어줘"]
        
        print(f"prompt_msg: {prompt_msg}")
        while True:
        # data = {}
            data = self.get_yt_info(url)
            print(f"data check: {data['title']} |  {data['video_id']} | {url}")
            exist_txt = self.task_file.exist_text_file(audio_id=audio_id)
            exist_path = self.task_file.exist_embed_file(audio_id=audio_id)

            if not exist_txt:
                print(f"rag not exist_txt: {exist_txt}")
                self.run(url, exist_txt=exist_txt)
            else:
                print(f"rag exist_txt: {exist_txt}")                
                class_name = self.get_class_name(channel_id)
                is_not_rag = self.task_baas.get_key_not_equal_to(class_name=class_name, keyName="is_rag", data=True)

                # 해당 디테일 테이블의 is_rag 체크
                object_id = self.get_detail_object_id_by_video_id(class_name=class_name, video_id=audio_id)
                one = self.task_baas.get_key_datas(class_name=class_name, keyName="objectId", data=object_id)
                one_results = one['results']
                if len(one_results) == 0:
                    print(f"one: {one_results}")
                    break
                
                if len(one_results) > 0:
                    data['objectId'] = object_id
                    one_data = one_results[0]
                    if 'is_rag' in one_data:
                        is_rag_complete = one_data['is_rag']
                        print(f"rag_run start |  is_rag_complete: {is_rag_complete} | object_id: {object_id} | video_id: {audio_id}")
                        if is_rag_complete:
                            break


                for index, text in enumerate(texts):
              
                    print(f"rag_run exist_path: {exist_path} || {audio_id} || {index}")
                    if 'results' in is_not_rag and len(is_not_rag['results']) == 0:
                        break

                    rag_chain = self.task_rag.get_chain(url= url, file_path=f"{self.embed_dir}/{audio_id}.txt", index=index)
                    
                    if not exist_path:
                        sleep(10)
                        self.run(url, exist_txt=exist_txt)
                                           
                    else:                        
                        res = rag_chain.invoke({"input": text, "index": index})
                        answer = res["answer"].replace("```json", "").replace("```", "")
                        if index == 0:
                            data["summary"] = answer
                        elif index == 1:
                            # print(f"books: {answer}")

                            # {"result":["미치게 만드는 브랜드","홈페이지 마케팅 플랜","모든 비즈니스는 브랜딩이다","원페이지 마케팅 플랜"]}
                            try:
                                if type(answer) == str:
                                    books = json.loads(answer)['result']
                                    book_infos =[]
                                    for book in books:
                                        print(f"book: {book}")
                                        book_info = self.agent_naver.book_search(book)
                                        book_infos.append(book_info)

                                    data["book_info"] = book_infos
                                else:
                                    data["book_info"] = []
                            except Exception as e:
                                print(f"Error: {e}")
                                data["book_info"] = []
                        elif index == 2:
                            data["persons"] = answer
                        elif index == 3:
                            data["titles"] = answer
                        elif index == 4:
                            data["hashtags"] = answer
                        elif index == 5:
                            data["questions"] = answer
                            data['update_date'] = datetime.now().strftime('%Y-%m-%d %H:%M:%S')
                            data['update_user'] = "AI"
                            data['is_rag'] = True

                        if 'is_rag' in data:
                            self.save_or_update_report(class_name=class_name, audio_id=audio_id, data=data)
                            self.task_file.delete_audio_file(audio_id=audio_id)
                            break

                    # if is_rag:
                    #     break
                # print(f"rag_run data: {data}")   
                

            sleep(5)        
        # except Exception as e:
        #     print(f"[rag_run] Error: {e}")
        #     return None
    
    