
import requests
from dotenv import dotenv_values
import os
import json

config = dotenv_values(".env")

class BassClsNameTask:
    def __init__(self):
        self.app_url = config["PARSE_APP_URL"]

    def getHeaders(self):
        try:
            app_id_key = "PARSE_APP_ID"
            api_key_key = "PARSE_REST_API_KEY"
            self.app_id = config[app_id_key]
            self.api_key = config[api_key_key]
            # print(f"self.app_id {self.app_id} || self.api_key {self.api_key}")
            headers = {
                "X-Parse-Application-Id": self.app_id,
                "X-Parse-REST-API-Key": self.api_key,
            } 
            return headers
        except Exception as e:
            print(f"getHeaders error {e}")
            return None

    def basicGet(self, class_name:str):
        return requests.get(self.app_url+"/parse/classes/"+class_name,headers=self.getHeaders())

    def basicOrderGet(self, class_name:str, orderKey:str):
        # print(f'self.app_url {self.app_url} || class_name {class_name} || orderKey {orderKey}')
        return requests.get(self.app_url+"/parse/classes/"+class_name+"?order="+orderKey,headers=self.getHeaders())      



    def basicPost(self, class_name:str, data:dict):
        # print(f"data {data}")
        # print(f"self.app_url {self.app_url} || class_name {class_name} || data {data}")
        return requests.post(self.app_url+"/parse/classes/"+class_name,headers=self.getHeaders(),json=data)
    
    def basicDelete(self, class_name:str, object_id:str):
        return requests.delete(self.app_url+"/parse/classes/"+class_name+"/"+object_id,headers=self.getHeaders())
    
    def basicPut(self, class_name:str, object_id:str, data:dict):
        # print(f"self.app_url {self.app_url} || class_name {class_name} || object_id {object_id} || data {data}")
        return requests.put(self.app_url+"/parse/classes/"+class_name+"/"+object_id,headers=self.getHeaders(),json=data)

    def exist_key_date(self, class_name:str, keyName:str, data:str):
        params = {"where" : json.dumps( {keyName:data}) }
        res = requests.get(self.app_url+"/parse/classes/"+class_name,headers=self.getHeaders(),params=params)
        code = res.status_code
        try:
            data = res.json()
            if code == 200 and data["results"] != []:
                return True
        except Exception as e:
            print(f"exist_key_date error {e}")
            return False
        
    def get_object_id(self, class_name:str, keyName:str, data:str):
        params = {"where" : json.dumps( {keyName:data}) }
        print(f"self.app_url {self.app_url} || class_name {class_name} || params {params}")
        res = requests.get(self.app_url+"/parse/classes/"+class_name,headers=self.getHeaders(),params=params)
        code = res.status_code
        try:
            if code == 200:
                data = res.json()
                return data['results'][0]['objectId']
            else:
                return None
        except Exception as e:
            print(f"get_object_id error {e}")
            return None
        

    def get_key_datas(self, class_name:str, keyName:str, data:str):
        params = {"where" : json.dumps( {keyName:data}) }
        res = requests.get(self.app_url+"/parse/classes/"+class_name,headers=self.getHeaders(),params=params)
        code = res.status_code
        try:
            data = res.json()
            return data
            
        except Exception as e:
            print(f"get_key_dataerror {e}")
            return None   
        
    def get_key_not_equal_to(self, class_name:str, keyName:str, data:str):
        params = {"where" : json.dumps( {keyName:{"$ne":data}}) }
        res = requests.get(self.app_url+"/parse/classes/"+class_name,headers=self.getHeaders(),params=params)
        code = res.status_code
        try:
            data = res.json()
            return data
            
        except Exception as e:
            print(f"get_key_dataerror {e}")
            return None     

    ## 한번에 삭제하는 REST API가 없으므로 아래와 같이 일일이 삭제처리    
    def delete_all_class(self, class_name:str):
        res_dailys = self.basicGet(class_name=class_name)
        list_data = []
        for res_daily in res_dailys.json()['results']:
            object_id = res_daily['objectId']
            res_delete = self.basicDelete(class_name=class_name, object_id=object_id )
            msg = f"삭제 {res_daily['author']} => {res_delete.status_code} || {object_id}"
            print(msg)
            list_data.append({'status': res_delete.status_code, 'msg': msg})
        return list_data

