import os
import scrapetube
from urllib.parse import urlparse
from pytubefix import YouTube, Channel
from pytubefix.innertube import InnerTube
from ..ha_constant import HaConstant

from selenium import webdriver
from selenium.webdriver.chrome.options import Options
from selenium.webdriver.common.keys import Keys
from selenium.webdriver.common.by import By

import time
import json

from dotenv import load_dotenv

env = os.getenv("ENV", "dev")
load_dotenv(f".env.{env}")

class HaCommonTask:
    def __init__(self):   
        token_file_folder = "src/digiloglab/token"
        self.token_file_folder = token_file_folder

    def getChannelInfo(self,url:str):
        channel = Channel(url=url)
        return channel
    
    def getYt(self,url:str):
        audio_id = self.get_audio_id(url)
        embed_url = f"https://www.youtube.com/embed/{audio_id}"
        yt = YouTube(url=embed_url, use_po_token=True, po_token_verifier=self.token_verifier)
        return yt
    
    def getYoutubeInfo(self,videourl:str):
        yt = self.getYt(url=videourl)
        try:
            channel_info = self.getChannelInfo(url=yt.channel_url)
            yt.channel_thumbnail = channel_info.thumbnail_url
            yt.channel_description = channel_info.description
        except Exception as e:
            print(e)
        return yt
    
    ## 유튜브 채널 재생목록 탭
    def getPlaylistInfo(self,channel_url:str):
        ch = self.getChannelInfo(url=channel_url)
        return ch.playlists
    
    ## 챕터
    def getChapterInfo(self,playlist_url:str):
        yt = self.getYt(url=playlist_url)
        return yt.chapters
    
    

    def get_channel_id(self, url:str) -> str:
        yt = self.getYoutubeInfo(url)
        return yt.channel_id
    
    def get_youtube_info(self, url:str) -> YouTube:
        yt = self.getYoutubeInfo(url)
        return yt
    
    def get_channel_urls(self, url:str, limit:int=10, sort_by:str= "newest", sleep:int=1) -> list[str]:
        channel_id = self.get_channel_id(url)
        videos = scrapetube.get_channel(channel_id=channel_id, limit=limit, sort_by=sort_by, sleep=sleep)
        urls = []
        for video in videos:
            url = 'https://www.youtube.com/'+video['navigationEndpoint']['commandMetadata']['webCommandMetadata']['url']
            urls.append(url)
        return urls

    def get_audio_id(self, url:str) -> str:
        urlp = urlparse(url)
        paras = urlp.query
        audio_id = paras.replace("v=", "")
        if "&" in audio_id:
            audio_id = audio_id.split("&")[0]
        else:
            audio_id = audio_id
        return audio_id
    
    def exist_stt_file(self, audio_id:str) -> bool:
        text_dir = HaConstant.text_dir
        file_path = f"{text_dir}/{audio_id}.txt"
        return os.path.exists(file_path)
    

    def regenerate_network_token(self, url:str):
        audio_id = self.get_audio_id(url)
        embed_url = f"https://www.youtube.com/embed/{audio_id}"
        chrome_options = Options()
        chrome_options.add_argument('--no-sandbox')
        chrome_options.add_argument("--headless")
        chrome_options.add_argument('--incognito')
        chrome_options.add_argument('--disable-dev-shm-usage')
        chrome_options.set_capability(
            'goog:loggingPrefs', {'performance': 'ALL'})

        driver = webdriver.Chrome(options=chrome_options)

        driver.get(url=embed_url)
        driver.delete_all_cookies()
        driver.refresh()
        driver.implicitly_wait(1)

        # press play btn
        elem = driver.find_element(
            By.CLASS_NAME, "ytp-large-play-button-red-bg")
        elem.send_keys(Keys.SPACE)
        time.sleep(1)

        # get network log
        perf = driver.get_log('performance')

        # filter log
        for entry in perf:
            log_message = json.loads(entry['message'])

            postData = log_message.get("message", {}).get(
                "params", {}).get("request", {}).get("postData", {})

            if "poToken" not in postData:
                continue

            postData = json.loads(postData)

            visitorId = postData['context']['client']['visitorData']
            poToken = postData['serviceIntegrityDimensions']['poToken']

        # write as json
        with open(f"{self.token_file_folder}/token.json", 'w', encoding='utf-8') as f:
            json.dump({"visitorData": visitorId, "po_token": poToken},
                    f, ensure_ascii=False, indent=4)

        driver.quit()

    def token_verifier(self):
        # with open(f"../token/token.json", 'r', encoding='utf-8') as f:
        #     token_file = json.load(f)
        # return token_file['visitorData'], token_file['po_token']
        return {
            "visitorData": "CgtpOThfZC00NDFhZyiNpeK7BjIKCgJLUhIEGgAgTQ%3D%3D",
            "po_token": "MncA2-8DV_K3ggDg3GLUppryyRh2Jfpw1cQC6Dt1azc4pnopFxYhum5YPyowpjeH5BEXJWQsC5k9VbLJVTHj-kLfjoL5lVA2N1o4NuO2uv3ANm7iWJAWqwNcveD3HBP-BOUpPjmnolCeEki2YyDJSQcYMyVLAFtW8w=="
        }
